import logging

LOG_FORMAT = "%(asctime)s-%(process)d-%(filename)s:%(lineno)d-%(levelname)s-%(message)s"


def set_log(verbose: int = 0):

    level_map = {
        0: logging.WARNING,
        1: logging.INFO,
        2: logging.DEBUG
    }

    level = level_map.get(verbose, logging.WARNING)

    logging.basicConfig(level=level, format=LOG_FORMAT)



import json
import random
import sys
import logging

from typing import List, Optional


from ..ip_detector import detectors
from tencentcloud.common.exception.tencent_cloud_sdk_exception import TencentCloudSDKException

# ---------- 常量 ----------
TEMPLATE_PREFIX = "EW-TEMPLATE-"
TEMPLATE_ID_PREFIX = "ipm-"
HEADER_WIDTH = 150
COLS = {
    "idx": 10,
    "id": 20,
    "ctime": 30,
    "addrs": 60,
    "name": 30,
}

# 这段是获取和打印模板


def _get_template(common_client) -> Optional[dict]:
    try:
        # templates = common_client.call_json("DescribeAddressTemplates", params, options = {"SkipSign": True})
        return common_client.call_json("DescribeAddressTemplates", {})

    except TencentCloudSDKException as e:
        logging.error("[template] DescribeAddressTemplates failed, %s", e)
        return None


def print_template(common_client) -> List[str]:

    if not (tpl_resp := _get_template(common_client)):
        return []

    template_ids = []

    # 表头
    header = (f"{"#":<{COLS["idx"]}}"
              f"{"Template ID":<{COLS["id"]}}"
              f"{"CreatedTime":<{COLS["ctime"]}}"
              f"{"Addresses":<{COLS["addrs"]}}"
              f"{"AddressTemplateName":<{COLS["name"]}}")

    print(f"{"Tencent Cloud Template List":=^{HEADER_WIDTH}}")
    print(header)
    print(f"-" * HEADER_WIDTH)

    for i, template in enumerate(tpl_resp["Response"]["AddressTemplateSet"], 1):
        template_ids.append(template["AddressTemplateId"])
        addreset = ", ".join(template["AddressSet"][:3])
        if len(template["AddressSet"]) > 3:
            addreset += f" ~~~ {len(template["AddressSet"])-3} more..."
        print(f"{str(i):{COLS["idx"]}}"
              f"{template["AddressTemplateId"]:{COLS["id"]}}"
              f"{template["CreatedTime"]:{COLS["ctime"]}}"
              f"{addreset:<{COLS["addrs"]}}"
              f"{template["AddressTemplateName"]:{COLS["name"]}}"
              )
    print("-" * HEADER_WIDTH)

    return template_ids


def _get_iplist(proxy=None):

    client_ip_list = detectors.get_local_ips(proxy)
    client_ip_list = list(set(client_ip_list))

    return client_ip_list


def _modify_template_address(common_client, target_id, client_ips):

    if not target_id:
        return False

    # 增加描述校验，避免更改错误
    params = {"Filters": [
        {"Name": "address-template-id", "Values": [target_id]}]}
    try:
        respon = common_client.call_json("DescribeAddressTemplates", params)
        if (TemplateSet := respon["Response"]["AddressTemplateSet"]) and TemplateSet[0]["AddressTemplateName"].startswith(TEMPLATE_PREFIX):
            pass
        else:
            logging.error("[template] this is not a template generated from this tool. Shall not be modified.")
            return False
    except (TencentCloudSDKException, IndexError) as err:
        # IndexError catch when there is no match target.Example: "AddressTemplateSet": []
        logging.error("[template] api call failed, reason=exception, detail=%s", err)
        sys.exit(1)

    params = {"AddressTemplateId": target_id,
              "AddressesExtra": [{"Address": ip, "Description": "client_ip"} for ip in client_ips]
              }

    try:
        respon = common_client.call_json(
            "ModifyAddressTemplateAttribute", params)

    except TencentCloudSDKException as err:
        logging.error("[template] api call failed, reason=exception, detail=%s", err)
        return False

    return True


def set_template(common_client, target_id, proxy=None):
    """更新模板 IP，返回是否成功"""
    if not target_id:
        logging.error("[template] missing template_id, reason=empty input")
        return False

    if not target_id.startswith(TEMPLATE_ID_PREFIX):
        logging.warning("[template] set failed, reason=wrong template id, hint=check prefix")
        return False

    client_iplist = _get_iplist(proxy)

    if _modify_template_address(common_client, target_id, client_iplist):
        print(f"✅ [成功] 模板 {target_id} 已更新 -> {client_iplist}")
        return True
    else:
        # 底层修改失败
        logging.error("[template] failed to update template %s", target_id)
        print(f"❌ [失败] 模板 {target_id} 更新失败（请检查网络或模板状态）")
        return False


def create_template_and_associate(common_client, rule_id, proxy=None):

    if not rule_id:
        logging.error("[template] security group ID required but missing")
        return False
    # check rule_id
    pass

    template_id, ret_val = create_template(common_client, proxy)

    if ret_val == 1:
        return True
    elif ret_val == 2:
        return associate_template_2_rule(common_client, template_id, rule_id)
    else:
        return False


def create_template(common_client, proxy=None):
    try:
        params = {"Filters": [{"Name": "address-template-name", "Values": [TEMPLATE_PREFIX]}]}
        respon = common_client.call_json("DescribeAddressTemplates", params)

        logging.debug("[template] API response, detail=%s", json.dumps(respon, ensure_ascii=False))

        if respon["Response"]["AddressTemplateSet"]:
            # 找到第一个符合的模板就设置
            existing_template = respon["Response"]["AddressTemplateSet"][0]
            template_id = existing_template["AddressTemplateId"]
            template_name = existing_template["AddressTemplateName"]

            logging.info("[template] already have template without creation: %s", template_id)

            print(f"🔄 [进行中] 已有模板 {template_id} ({template_name})，直接在模板更新本地公网IP")

            set_template(common_client, template_id)
            return template_id, 1

        ip_list = _get_iplist(proxy)
        random_suffix = random.randint(1, 9999)
        template_name = f"{TEMPLATE_PREFIX}{random_suffix:04d}"
        params = {
            "AddressTemplateName": template_name,
            "AddressesExtra": [{"Address": ip, "Description": "client_ip"} for ip in ip_list]
        }
        print(f"🎯 [开始] 创建模板, 模板名字为：{template_name}")

        respon = common_client.call_json("CreateAddressTemplate", params)
        template_id = respon["Response"]["AddressTemplate"]["AddressTemplateId"]
        logging.info("[template] API response, detail=%s", json.dumps(respon, ensure_ascii=False))

        print(f"🔄 [进行中] 模板 {template_id} 已创建")

        return template_id, 2

    except TencentCloudSDKException as err:
        logging.error("[template] API failed, reason=exception, detail=%s", err)
        return None, 3


def associate_template_2_rule(common_client, template_id, rule_id):

    try:
        # 避免重复关联
        params = {
            "SecurityGroupId": rule_id,
            "Filters": [{"Name": "address-module", "Values": [template_id]}]
        }
        respon = common_client.call_json("DescribeSecurityGroupPolicies", params)

        if respon["Response"]["SecurityGroupPolicySet"]["Ingress"]:
            logging.info("[template] %s already associate to %s", template_id, rule_id)
            print(f"❗ [中止] 已有属于程序创建的模板 {template_id} 关联到 {rule_id}，仅允许关联一次")
            return False

        # 进入规则设置
        params = {"SecurityGroupId": f"{rule_id}",
                  "SecurityGroupPolicySet":
                  {"Ingress": [
                      {"PolicyIndex": 0, "Protocol": "ALL", "AddressTemplate": {
                          "AddressId": f"{template_id}"},
                       "Action": "ACCEPT", "PolicyDescription": "Easy Whitelist"}
                  ]}
                  }

        respon = common_client.call_json("CreateSecurityGroupPolicies", params)
        logging.info("[template] API response, detail=%s", json.dumps(respon, ensure_ascii=False))
        print(f"✅ [成功] 模板 {template_id} 已关联到 {rule_id}")

    except TencentCloudSDKException as err:
        logging.error("[template] api failed, reason=exception, detail=%s", err)
        return False

    return True



import argparse


def _port(txt: str) -> int:
    """argparse type checker: 1-65535"""
    n = int(txt)
    if not 0 < n < 65536:
        raise argparse.ArgumentTypeError(f"Port must be 1-65535, got {n}")
    return n


def init_arg():
    """Parse CLI for ew (cloud ACL auto-whitelist)."""

    parser = argparse.ArgumentParser(
        prog="ew",
        description="This is a cloud acl auto whitelist tool.",
        epilog="Enjoy the tool. :) ")

    # 云厂商互斥组
    cloud_grp = parser.add_mutually_exclusive_group(required=False)

    cloud_grp.add_argument(
        "-t", "--tencent", action="store_const", const="tencent", dest="cloud",
        help="use Tencent Cloud (default)"
    )
    cloud_grp.add_argument(
        "-a", "--alibaba", action="store_const", const="alibaba", dest="cloud",
        help="use Alibaba Cloud"
    )
    parser.set_defaults(cloud="tencent")  # 显式给默认值

    # 可选参数
    parser.add_argument("-p", "--proxy", type=_port,
                        metavar="port", help="local HTTP proxy port")

    parser.add_argument("-v", "--verbose", action="count", default=0)

    # 位置参数
    parser.add_argument("-r", "--region", help="region or rule")

    parser.add_argument("target", help="template or rule",
                        choices=["template", "rule"])

    parser.add_argument("action", help="action to perform", choices=[
                        "list", "set", "create"])

    parser.add_argument("target_id", help="template id or rule id", nargs="?")

    args = parser.parse_args()

    return args



import logging

from .config import arg
from .config.log import set_log
from .tcloud.core import t_main


def main() -> None:

    # tencent, alibaba, action, target, target_id, proxy, verbose = arg.init_arg()
    args = arg.init_arg()

    set_log(args.verbose)
    logging.info("[cli] arg parsed, detail=%s", args)

    cloud_provider = args.cloud
    logging.info("[cli] cloud provider selected, provider=%s", cloud_provider.upper())

    if cloud_provider == "tencent":
        t_main(args.action, args.target, args.target_id,  args.region, args.proxy)



import requests
import random
import string
import time

'''
1. 密码没有安全策略
2. 密码没有设置过初始密码，但是后台看起来设置过初始密码
3. 密码传输在URL里面
'''

def rand_password(n=16, alphabet=string.ascii_letters + string.digits):
    return ''.join(random.choices(alphabet, k=n))


def main() -> None:
    old_p = 'ny1Pol'
    new_p = rand_password(6)

    for i in range(100):
        print(old_p, new_p)
        u = f'https://wxe0809d0b720d56b4.wx.gcihotel.net/guardian/api/member/info.json?hotelGroupId=2&hotelGroupCode=SOOCOR&memberId=2025071849927431&newPassword={new_p}&oldPassword={old_p}'
        ret = requests.post(u)
        print(ret.text)

        old_p = new_p
        new_p = rand_password(6)

        time.sleep(1)

if __name__ == '__main__':
    main()
    
# for i in range(100):
#     requests.get()



import logging
from enum import Enum, auto
from typing import Optional

from .template import print_template, set_template


class CommandAction(Enum):
    CONTINUE = auto()
    BREAK = auto()
    NOTHING = auto()


CMD_LIST = "l"
CMD_EMPTY = ""
CMD_CREATE = "c"
CMD_EXIT = "q"
INPUT_PROMPT = "Please choose # template to set (or [L]ist, [C]reate, [Q]uit): "


def _handle_digit_input(user_input: str, common_client, template_ids: list, proxy: Optional[str]) -> None:
    """
    处理数字输入，选择模板索引

    Args:
        user_input: 用户输入的数字字符串
        common_client: 云服务客户端
        template_ids: 模板ID列表
        proxy: 代理设置，可选
    """

    if not template_ids:
        logging.warning("[template] no template available, reason=no template, hint=create one first")
        return

    try:
        index = int(user_input)
        if 1 <= index <= len(template_ids):
            set_template(common_client, template_ids[index - 1], proxy)
        else:
            logging.warning("[template] select failed, reason=index out of range, hint=available 1~%d", len(template_ids))
    except ValueError:
        logging.warning("[template] select failed, reason=invalid number %s, hint= %d", user_input, len(template_ids))


def _handle_command_input(user_input: str, common_client, template_ids: list, proxy: Optional[str]) -> CommandAction:
    """
    处理命令输入，执行相应操作

    Args:
        user_input: 用户输入的命令
        common_client: 云服务客户端
        template_ids: 模板ID列表
        proxy: 代理设置，可选

    Returns:
        CommandAction: 指示后续操作的动作
    """

    command_handlers = {
        CMD_LIST: (lambda: print_template(common_client), CommandAction.CONTINUE),
        CMD_EMPTY: (lambda: None, CommandAction.CONTINUE),
        CMD_CREATE: (lambda: None, CommandAction.CONTINUE),
        CMD_EXIT: (lambda: None, CommandAction.BREAK),
        # "c": lambda: create_template(common_client, proxy) # 假设有create_template函数
        # 可轻松扩展其他命令，例如 "h": show_help
    }

    if user_input in command_handlers:
        handler, action = command_handlers[user_input]
        handler()
        return action
    else:
        logging.warning("[cli] command failed, reason=invalid command %s, hint=l/c/q", user_input)
        return CommandAction.CONTINUE


def loop_list(common_client, proxy: Optional[str] = None) -> None:
    template_ids = print_template(common_client)
    last_input = None

    while True:

        try:
            user_input = input(INPUT_PROMPT).strip().lower()

            if last_input == "" and user_input == "":
                break

            last_input = user_input

            if user_input.isdigit():
                _handle_digit_input(
                    user_input, common_client, template_ids, proxy)
            else:
                action = _handle_command_input(
                    user_input, common_client, template_ids, proxy)
                if action == CommandAction.BREAK:
                    break

        except KeyboardInterrupt:
            logging.warning("[cli] operation cancelled, reason=user interrupt, hint=none")
            break

        except ValueError as e:
            logging.warning("[cli] input failed, reason=value error %s, hint=retry", e)

        except ConnectionError as e:
            logging.error("[http] connection failed, reason=connection error, detail=%s", e)
            break

        except Exception as e:
            logging.error("[http] request failed, reason=unexpected, detail=%s", e)
            break



import requests
import re
import logging

from . import utils


def get_local_ip_from_url_and_parse(u, patt, ag, if_enable, proxy=None):
    # 发送GET请求
    headers = {"user-agent": ag}

    if not re.search("enable", if_enable, re.IGNORECASE):
        return None

    try:
        logging.info("[ip.detect] fetching local IP, url=%s proxy=%s", u, proxy)

        if proxy:
            response = requests.get(u, headers=headers, timeout=(3, 5), proxies={"http": f"http://127.0.0.1:{proxy}", "https": f"http://127.0.0.1:{proxy}"})
        else:
            response = requests.get(u, headers=headers, timeout=(3, 5))

        # 获取响应内容
        respon = response.text
        l_ip = utils.parse_ip_from_response(respon, patt)
        logging.info("[ip.detect] fetched local IP, url=%s ip=%s", u, l_ip)

        return l_ip
    except Exception as e:
        logging.error("[ip.detect] parse failed, reason=exception, detail=%s", e)
        return None


def validate_ip(l_ip):
    if not l_ip:
        return False

    # r"(?:(?:25[0-5]|2[0-4][0-9]|[1]?[0-9][0-9]?)\.){3}(?:25[0-5]|2[0-4][0-9]|[1]?[0-9][0-9]?)"
    # r"(?:\d{1,3}\.){3}\d{1,3}"
    # r"((?:[1-9]?\d|1\d\d|2[0-4]\d|25[0-5])\.){3}(?:[1-9]?\d|1\d\d|2[0-4]\d|25[0-5])"
    # r"(?<![\.\d])(?:25[0-5]\.|2[0-4]\d\.|[01]?\d\d?\.){3}(?:25[0-5]|2[0-4]\d|[01]?\d\d?)(?![\.\d])"
    pat = r"((?:[1-9]?\d|1\d\d|2[0-4]\d|25[0-5])\.){3}(?:[1-9]?\d|1\d\d|2[0-4]\d|25[0-5])"
    if re.fullmatch(pat, l_ip):
        return True
    else:
        return False


def get_local_ips(proxy=None):
    ip_list = []
    for i, u in enumerate(utils.detect_url, 1):
        l_ip = get_local_ip_from_url_and_parse(u[0], u[1], u[2], u[3], proxy)
        if l_ip and validate_ip(l_ip):
            ip_list.append(l_ip)
    return ip_list


def print_ip_list(ip_list):
    number = 100
    print(f"{"Detected Local IP List":=^{number}}\n"
          f"{"#":<38}IP Address\n"
          f"{"-" * number}"
          )

    for i, ip in enumerate(ip_list, 1):
        print(f"{str(i):<38}{ip}")

    print("-" * number)



import logging

from . import client
from .template import set_template, create_template_and_associate
from .ls_template import loop_list


def t_main(action, target, target_id, region, proxy=None) -> None:
    if target == "template":
        common_client = client.get_common_client(proxy, region)

        ACTION_MAP = {
            "list": lambda: loop_list(common_client, proxy),
            "set": lambda: set_template(common_client, target_id, proxy),
            "create": lambda: create_template_and_associate(common_client, target_id, proxy),
        }

        if action in ACTION_MAP:
            ACTION_MAP[action]()
        else:
            logging.error("[cli] unsupported operation, reason=unknown action, detail=%s", action)
    else:
        logging.error("[cli] unsupported target, reason=not implemented, detail=%s", target)



import os
import logging

from tencentcloud.common import credential
from tencentcloud.common.profile.http_profile import HttpProfile
from tencentcloud.common.profile.client_profile import ClientProfile
from tencentcloud.common.common_client import CommonClient


def get_common_client(proxy_port, region):
    # cred = credential.Credential(
    #     os.environ.get("TENCENTCLOUD_SECRET_ID"),
    #     os.environ.get("TENCENTCLOUD_SECRET_KEY"))

    if region is None:
        region = "ap-guangzhou"
        logging.info("[config] region not set, fallback to %s", region)

    cred = credential.DefaultCredentialProvider().get_credential()

    httpProfile = HttpProfile()
    httpProfile.endpoint = "vpc.tencentcloudapi.com"
    httpProfile.proxy = f"127.0.0.1:{proxy_port}" if proxy_port is not None else None

    clientProfile = ClientProfile()
    clientProfile.httpProfile = httpProfile
    # clientProfile.signMethod = "HmacSHA256"

    common_client = CommonClient(
        "vpc", "2017-03-12", cred, region, profile=clientProfile)

    return common_client



import re
import random

safari_user_agent = [
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/17.5 Safari/605.1.15"]
chrome_user_agent = [
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/127.0.0.0 Safari/537.36"]
edge_user_agent = [
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/127.0.0.0 Safari/537.36 Edg/127.0.0.0"]
curl_user_agent = ["curl/8.6.0", "curl/7.29.0"]

IFCONFIG_ME_PATTERN = r"(.+)"
CIP_CC_PATTERN = r"URL\s+?:\s+?http://www.cip.cc/(.+)"
TOOL_LU_PATTERN = r"<p>你的外网IP地址是：\s*?(.+)</p>"
IP_SB_PATTERN = r"(.+)"


def parse_ip_from_response(response, patt):
    if result := re.search(patt, response):
        return result.group(1)


detect_url = [
    ["https://ifconfig.me", IFCONFIG_ME_PATTERN,
        random.choice(curl_user_agent), "enable"],
    ["http://cip.cc", CIP_CC_PATTERN,
        random.choice(chrome_user_agent), "DISABLE"],
    ["https://tool.lu/ip/", TOOL_LU_PATTERN,
        random.choice(chrome_user_agent), "enable"],
    ["http://ip.sb/", IP_SB_PATTERN, random.choice(curl_user_agent), "enable"]
]
